"""Importing standart libs"""
from flask import Flask
from flask_restful import Api
import inspect, os

from restrunner.resources.query import CommandList
from restrunner.resources.query import SearchTask
from restrunner.resources.restrunner import RunCommand

from restrunner.common import data
from restrunner.common import db
from restrunner.conf import settings

# Setup flask rest
app = Flask(__name__)
api = Api(app)

class Help():
    """Flask Restfull class for Help."""
    def get(self):
        """Flask restfull standart get function."""
        routes = []
        for route in app.url_map.iter_rules():
            routes.append(str(route))
        return routes


# Setup the Api resource routing help
#api.add_resource(Help, '/help')



# Flask automatically creates a /static/<path:filename> route
#that will serve any filename under the static folder


# Setup the Api resource routing realated commands
api.add_resource(CommandList, '/commands')
api.add_resource(RunCommand, '/commands/run/<name>')


# Setup the Api resource routing realated tasks
api.add_resource(SearchTask, '/tasks/<int:status>')

class RestRunner:
    __conf = {
        "port": 5000,
        "debug": False,
        "host": '127.0.0.1',
        "data_folder": ""
    }
    __setters = ["port", "debug", "host", "data_folder"]

    @staticmethod
    def config(name):
        return RestRunner.__conf[name]

    @staticmethod
    def set(name, value):
        if name in RestRunner.__setters:
            RestRunner.__conf[name] = value
        else:
            raise NameError("Name not accepted in set() method")

    def run(self):

        settings.PORT = RestRunner.__conf["port"]
        settings.DEBUG = RestRunner.__conf["debug"]
        settings.HOST = RestRunner.__conf["host"]
        settings.DATA_FOLDER = RestRunner.__conf["data_folder"]

        if(settings.DATA_FOLDER == ""):
            runner_path = os.path.abspath(inspect.stack()[1][1])
            runner_path = runner_path[0:runner_path.rindex("/")]
            settings.DATA_FOLDER = runner_path+"/rr-data"
            data.create_data_folder(settings.DATA_FOLDER)
        else:
            settings.DATA_FOLDER = settings.DATA_FOLDER.removesuffix('/')+"/rr-data"
            data.create_data_folder(settings.DATA_FOLDER)

        db.init()
        app.run(debug=settings.DEBUG,
                port=settings.PORT,
                host=settings.HOST)



"""Import tinydb and datetime"""
from tinydb import TinyDB, Query
from restrunner.conf import settings as Settings
import os

commands_json_file = None
tasks_json_file = None
commands = None
tasks = None
q = None

from restrunner.common import db

def init():
    db.commands_json_file = Settings.DATA_FOLDER+"/commands.json"
    db.tasks_json_file = Settings.DATA_FOLDER+"/tasks.json"

    db.commands = TinyDB(commands_json_file, sort_keys=True, indent=4, separators=(',', ': '))
    fpath = Settings.DATA_FOLDER+"/commands.json"
    if os.stat(fpath).st_size == 0:
        db.commands.insert_multiple(Settings.DEFAULT_COMMANDS)
    
    db.tasks = TinyDB( tasks_json_file, sort_keys=True, indent=4, separators=(',', ': '))
    db.q = Query()

# Command info operations

def get_all_commands():
    """Return all commands in db.json in list type."""
    print("------------"+str(commands.all()))
    return commands.all()

def search_command(value):
    """Search commands in commands table in db.json."""
    res = commands.search(q.name == value)
    if len(res) != 0 :
        return res[0]
    return 0

# Task operations

def search_task(status):
    """Search task in logs table in db.json with status.
    Status parameter can be only 0,1,2.
    0=fail, 1=success, 2=continuing"""
    res = tasks.search(q.status == status)
    print(res)
    if len(res) != 0 :
        return res
    return 0

def new_task(items):
    """Insert new task item like this
    item = {
        "fdate": "2022-10-21-16-58-59",
        "file": "log/2022-10-21-16-58-54-test4.log",
        "name": "test4",
        "sdate": "2022-10-21-16-58-54",
        "status": 1
    }"""
    return tasks.insert(items)

def update_task(id, name, value):
    tasks.update({name: value}, doc_ids=[id] )

def get_all_tasks():
    """Return all commands in db.json in list type."""
    return tasks.all()



from flask_restful import Resource
from flask_restful import abort
from flask import jsonify
from threading import Thread

from restrunner.common import db
from restrunner.common.cmd import run

def abort_doesnt_exist(name):
    """Abort request if doesnt exist."""
    abort(404, message=f"{name} doesn't exist")

class RunCommand(Resource):
    """Flask Restfull class for running command."""

    def get(self, name):
        """Flask restfull standart get function."""
        val = db.search_command(name)
        if val != 0 :
            thread = Thread(target=run, args=(val['name'], val['command']))
            thread.start()
            return jsonify(result=True)
        else:
            abort_doesnt_exist(name)